#!/usr/bin/python
# Copyright 2016 CERN and GSI
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from iecommon import *
from model.Class.Register import *
import libxml2


class Block(object):

    ___settingBlockType = "Setting-Block"
    ___acquisitionBlockType = "Acquisition-Block"
    ___commandBlockType = "Command-Block"
    
    def __init__(self, xmlNode):
        self.xmlNode = xmlNode
        self.name = xmlNode.prop("name")
        self.___type = xmlNode.get_name()
                #xmlNode.shellPrintNode()

    def getNameCapitalized(self):
        return  iecommon.capitalizeString(self.name)
    
    def isReadable(self):
        return self.___type == self.___settingBlockType or self.___type == self.___acquisitionBlockType
    
    def isWritable(self):
        return self.___type == self.___settingBlockType or self.___type == self.___commandBlockType

    def isAcquisition(self):
        return self.___type == self.___acquisitionBlockType
    
    def isSetting(self):
        return self.___type == self.___settingBlockType
    
    def isCommand(self):
        return self.___type == self.___commandBlockType
    
    def getRegisterNodes(self):
        return self.xmlNode.xpathEval("*[name()='Acquisition-Register' or name()='Setting-Register' or name()='Volatile-Register']")
    
#has some additionalValues
class ParamBlock(Block):
    def __init__(self):
        self.size = 0
        self.address = 0
        self.memSize = 0

            
    def initWithParamBlockNode(self, xmlNode):
        super(ParamBlock, self).__init__(xmlNode)
        self.size = long(self.xmlNode.prop("size"))
        self.address = long(self.xmlNode.prop("address"))
        self.memSize = long(self.xmlNode.prop("mem-size"))
        
    def initWithDesignBlock(self, designBlock):
        newNode = libxml2.newNode(designBlock.xmlNode.get_name())
        newNode.newProp("name", designBlock.name)
        super(ParamBlock, self).__init__(newNode)
        newNode.newProp("size", str(self.size))
        newNode.newProp("address", str(self.address))
        newNode.newProp("mem-size", str(self.memSize))
        
    def setSize(self,size):
        self.xmlNode.setProp("size", str(size))
        self.size = size
        
    def setAddress(self,address):
        self.xmlNode.setProp("address", str(address))
        self.address = address
        
    def setMemSize(self,memSize):
        self.xmlNode.setProp("mem-size", str(memSize))
        self.memSize = memSize
        
    def getParamRegisters(self):
        paramRegisters = []
        for registerNode in self.getRegisterNodes():
            paramRegister = ParamRegister()
            paramRegister.initWithParamRegisterNode(registerNode)
            paramRegisters.append(paramRegister)
        return paramRegisters
    
                
class DesignBlock(Block):
            
    def __init__(self, xmlNode):
        super(DesignBlock, self).__init__(xmlNode)
        self.generateFesaProperty = False
        self.fesaPropertyName = ""
        self.fesaGetServerActionName = ""
        self.fesaSetServerActionName = ""
    
        if self.xmlNode.hasProp("fesaPropertyName"):
            self.fesaPropertyName = xmlNode.prop("fesaPropertyName")
        else:
            self.fesaPropertyName = self.name
            
        if self.xmlNode.hasProp("generateFesaProperty"): #SilecsHEader does not have this attribute 
            self.generateFesaProperty = xsdBooleanToBoolean(xmlNode.prop("generateFesaProperty"))

        if self.xmlNode.hasProp("fesaGetServerActionName"):
            self.fesaGetServerActionName = xmlNode.prop("fesaGetServerActionName")
        else:
            self.fesaGetServerActionName = 'Get' + self.fesaPropertyName

        if self.xmlNode.hasProp("fesaSetServerActionName"):
            self.fesaSetServerActionName = xmlNode.prop("fesaSetServerActionName")
        else:
            self.fesaSetServerActionName = 'Set' + self.fesaPropertyName

    def getFesaName(self):
        return self.fesaPropertyName
    
    def getDesignRegisters(self):
        designRegisters = []
        for registerNode in self.getRegisterNodes():
            designRegisters.append(DesignRegister(registerNode))
        return designRegisters
