#!/usr/bin/python
# Copyright 2016 CERN and GSI
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from iecommon import *
from model.Class.Block import *
import libxml2

class Class(object):
    
    def __init__(self, xmlNode):
        self.xmlNode = xmlNode
        self.name = xmlNode.prop("name")
        self.version = xmlNode.prop("version")
        #xmlNode.shellPrintNode()

    def getNameCapitalized(self):
        return  iecommon.capitalizeString(self.name)

    def getBlockNodes(self):
        return self.xmlNode.xpathEval("*[name()='Acquisition-Block' or name()='Setting-Block' or name()='Command-Block']")
    
class ParamClass(Class):
    def __init__(self):
        self.address = 0
        self.usedMemory = ""

            
    def initWithParamClassNode(self, xmlNode):
        super(ParamClass, self).__init__(xmlNode)
        self.address = long(self.xmlNode.prop("address"))
        self.usedMemory = self.xmlNode.prop("usedMemory")
        
    def initWithDesignClass(self, designClass):
        newNode = libxml2.newNode(designClass.xmlNode.get_name())
        newNode.newProp("name", designClass.name)
        newNode.newProp("version", designClass.version)
        super(ParamClass, self).__init__(newNode)
        newNode.newProp("address", str(self.address))
        newNode.newProp("usedMemory", self.usedMemory)
        
    def setAddress(self,address):
        self.xmlNode.setProp("address", str(address))
        self.address = address
        
    def setUsedMemory(self,usedMemory):
        self.xmlNode.setProp("usedMemory", usedMemory)
        self.usedMemory = usedMemory
        
    def getParamBlocks(self):
        paramBlocks = []
        for blockNode in self.getBlockNodes():
            paramBlock = ParamBlock()
            paramBlock.initWithParamBlockNode(blockNode)
            paramBlocks.append(paramBlock)
        return paramBlocks
    
    def getDeviceInstanceNodes(self):
        deviceNodes = self.xmlNode.xpathEval("Instance")
        return deviceNodes
    
    @staticmethod
    def getParamClassesFromRootNode(silecsRoot):
        classNodes = silecsRoot.xpathEval("/SILECS-Param/SILECS-Mapping/SILECS-Class")
        if len(classNodes) < 1:
            raise BaseException("Error: no class-node found in design-document")
        paramClasses = []
        for classNode in classNodes:
            paramClass = ParamClass()
            paramClass.initWithParamClassNode(classNode)
            paramClasses.append(paramClass)
        return paramClasses
    
                
class DesignClass(Class):
    def __init__(self, xmlNode):
        super(DesignClass, self).__init__(xmlNode)
        self.fesaPropertyName = self.name

    def getFesaName(self):
        return self.fesaPropertyName
    
    def getDesignBlocks(self):
        designBlocks = []
        for blockNode in self.getBlockNodes():
            designBlocks.append(DesignBlock(blockNode))
        return designBlocks
    
    @staticmethod
    def getDesignClassFromRootNode(silecsRoot):
        classNodes = silecsRoot.xpathEval('/SILECS-Design/SILECS-Class')
        if len(classNodes) > 1:
            raise "Error: multiple class-nodes found in design-document"
        if len(classNodes) < 1:
            raise "Error: no class-node found in design-document"
        return DesignClass(classNodes[0])
